/***************************************************************************
 *   Copyright (C) 2005 by Joel R. Voss                                    *
 *   jvoss@myuw.net                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/*
Robot Cat 1 will be designed to the AltSci ABCD robot specification.

Input A relates to known systems B <-> model C -> output D.

A input (audio, video, proximity)
B known systems (object, person, robot)
C model (love, hate tennis, soccer, etc)
D output (knowledge, action)

A to B: recognition (speech, image)
B to C: understanding (contest)
C to D: decision (purpose)

Robot Cat 1 will use the following technologies to implement ABCD.
recognition: neural networking
understanding: optimized contest
decision: purpose system
*/


#include "robotcat.h"

void RobotCat_init(RobotCat *cat, int inputs, int layers)
{
	// TODO: give the cat an identifier.
	cat->id   = 0;
	// TODO: give the cat a friendly name.
	cat->name = 0;
	// TODO: give the cat a quick reference lookup.
	cat->knowledge = 0;
	// Initialize the neural network to turn input into known models.
	NeuralNetwork_init(&(cat->recognition), inputs, layers);
}

void RobotCat_destroy(RobotCat *cat)
{
	cat->id = 0;
	free(cat->name);
	free(cat->knowledge);
	NeuralNetwork_destroy(&(cat->recognition));
}

NeuralInput *RobotCat_update(RobotCat *cat)
{
	// Things that Robot Cat can do:
	
	// TODO: Get input
	// TODO: input should ensure that there are the same number of input 
	// nodes as there are inputs.
	NeuralInput input[] = {true, false, true, false, true, true, false, 
		true, true, false, true};
	
	// Put it into the network.
	NeuralInput *output = NeuralNetwork_update(&(cat->recognition), input);
	
	// TODO: Learn from the input.
	
	// Return the output properly.
	return output;
}



